ChatSounds = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0")
local ChatSounds = ChatSounds
local self = ChatSounds
local dew = AceLibrary("Dewdrop-2.0")

local sounds = {
	TellMessage = "TellMessage",
	Heart = "Interface\\AddOns\\ChatSounds\\Sounds\\Heart.wav",
	Kachink = "Interface\\AddOns\\ChatSounds\\Sounds\\Kachink.wav",
	Link = "Interface\\AddOns\\ChatSounds\\Sounds\\Link.wav",
	Text1 = "Interface\\AddOns\\ChatSounds\\Sounds\\Text1.wav",
	Text2 = "Interface\\AddOns\\ChatSounds\\Sounds\\Text2.wav",
}
local strsub, strfind, strlower = strsub, strfind, strlower
local select = select
local db, opts, updatelist
local me

function ShowOptions()
	dew:Open( UIParent, 'children', function() dew:FeedAceOptionsTable(opts) end, 'cursorX', true, 'cursorY', true )
end

function ChatSounds:OnInitialize()
	self:RegisterDB("ChatSoundsDB")
	self:RegisterDefaults("profile", {
		incoming = {
			GUILD = "Kachink",
			OFFICER = "Link",
			PARTY = "Text1",
			RAID = "Text2",
			WHISPER = "Heart",
		},
		outgoing = {
			GUILD = "none",
			OFFICER = "none",
			PARTY = "none",
			RAID = "none",
			WHISPER = "none",
		},
		customlist = {},
	} )
	db = self.db.profile
	if db.ver ~= 2 then
		local tmp = {}
		for k,v in pairs(db.customlist) do
			tmp[v] = true
			db.customlist[k] = nil
		end
		for value in pairs(tmp) do
			db.customlist[value] = true
		end
		db.ver = 2
	end
	updatelist()
	me = UnitName("player")
	
	self:RegisterEvent("CHAT_MSG_CHANNEL")
	
	self:RegisterEvent("CHAT_MSG_GUILD", "GroupChat")
	self:RegisterEvent("CHAT_MSG_OFFICER", "GroupChat")
	self:RegisterEvent("CHAT_MSG_PARTY", "GroupChat")
	self:RegisterEvent("CHAT_MSG_RAID", "GroupChat")
	self:RegisterEvent("CHAT_MSG_RAID_LEADER", "GroupChat")
	self:RegisterEvent("CHAT_MSG_BATTLEGROUND", "GroupChat")
	self:RegisterEvent("CHAT_MSG_BATTLEGROUND_LEADER", "GroupChat")
	
	self:RegisterEvent("CHAT_MSG_MONSTER_WHISPER")
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
	
  	SlashCmdList["CHATSOUNDS"] = ShowOptions
   	SLASH_CHATSOUNDS1 = "/chatsounds"
end

local function Play(sound)
	if sound ~= "TellMessage" then
		PlaySoundFile(sounds[sound] or "TellMessage")
	else
		PlaySound("TellMessage")
	end
end

-- sounds for custom channels
function ChatSounds:CHAT_MSG_CHANNEL(...)
	local channame = strlower(select(9, ...))
	if select(2, ...) ~= me then
		for value in pairs(db.customlist) do
			if strfind(channame, strlower(value)) then
				Play(db.incoming.OFFICER)
			end
		end
	else
		for value in pairs(db.customlist) do
			if strfind(channame, strlower(value)) then
				Play(db.outgoing.OFFICER)
			end
		end
	end
end

-- sounds for party, raid (leader and bg included), guild, and officer
local raid = {
	RAID_LEADER = "RAID",
	BATTLEGROUND = "RAID",
	BATTLEGROUND_LEADER = "RAID",
}
function ChatSounds:GroupChat(a1,a2)
	local msgtype = strsub(event, 10)
	msgtype = raid[msgtype] or msgtype
	if a2 ~= me then
		Play(db.incoming[msgtype])
	else
		Play(db.outgoing[msgtype])
	end
end

function ChatSounds:CHAT_MSG_MONSTER_WHISPER()
	Play(db.incoming.WHISPER)
end

-- sounds for incoming whispers
function ChatSounds:CHAT_MSG_WHISPER(a1)
	-- La Vendetta has that annoying whisper function, so don't play a sound for those
	if strsub(a1, 1, 4) == "LVPN" or strsub(a1, 1, 4) == "LVBM" or strsub(a1, 1, 1) == "/" then return end
	Play(db.incoming.WHISPER)
end

-- sounds for outgoing whispers
function ChatSounds:CHAT_MSG_WHISPER_INFORM(a1)
	if strsub(a1, 1, 4) == "LVPN" or strsub(a1, 1, 4) == "LVBM" or strsub(a1, 1, 1) == "/" then return end
	Play(db.outgoing.WHISPER)
end


-- OPTIONS

do
	local list = {}
	function updatelist()
		for k,v in ipairs(list) do
			list[k] = nil
		end
		for entry in pairs(db.customlist) do
			tinsert(list, entry)
		end
		sort(list)
	end
	local function setinc(k,v)
		db.incoming[k] = v
	end
	local function setout(k,v)
		db.outgoing[k] = v
	end
	local function setinsert(v)
		db.customlist[v] = true
		updatelist()
	end
	local function setremove(v)
		db.customlist[v] = nil
		updatelist()
	end
	local function getinc(k)
		return db.incoming[k]
	end
	local function getout(k)
		return db.outgoing[k]
	end
	local soundslist = { "TellMessage", "Heart", "Kachink", "Link", "Text1", "Text2", "none"}
	opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "ChatSounds",
				order = 1,
			},
			inc = {
				type = "group",
				name = "Incoming Sounds", desc = "Sound selection for incoming chat messages",
				order = 2,
				args = {
					party = {
						type = "text",
						name = "Party", desc = "Sound for incoming party messages",
						set = setinc, get = getinc, passValue = "PARTY",
						order = 1,
						validate = soundslist,
					},
					raid = {
						type = "text",
						name = "Raid", desc = "Sound for incoming raid or battleground group/leader messages",
						set = setinc, get = getinc, passValue = "RAID",
						order = 2,
						validate = soundslist,
					},
					guild = {
						type = "text",
						name = "Guild", desc = "Sound for incoming guild messages",
						set = setinc, get = getinc, passValue = "GUILD",
						order = 3,
						validate = soundslist,
					},
					officer = {
						type = "text",
						name = "Officer/Custom", desc = "Sound for incoming officer or custom channel messages",
						set = setinc, get = getinc, passValue = "OFFICER",
						order = 4,
						validate = soundslist,
					},
					whisper = {
						type = "text",
						name = "Whisper", desc = "Sound for incoming whisper messages",
						set = setinc, get = getinc, passValue = "WHISPER",
						order = 5,
						validate = soundslist,
					},
				},
			},
			out = {
				type = "group",
				name = "Outgoing Sounds", desc = "Sound selection for outgoing (from you) chat messages",
				order = 3,
				args = {
					party = {
						type = "text",
						name = "Party", desc = "Sound for outgoing party messages",
						set = setout, get = getout, passValue = "PARTY",
						order = 1,
						validate = soundslist,
					},
					raid = {
						type = "text",
						name = "Raid", desc = "Sound for outgoing raid or battleground group/leader messages",
						set = setout, get = getout, passValue = "RAID",
						order = 2,
						validate = soundslist,
					},
					guild = {
						type = "text",
						name = "Guild", desc = "Sound for outgoing guild messages",
						set = setout, get = getout, passValue = "GUILD",
						order = 3,
						validate = soundslist,
					},
					officer = {
						type = "text",
						name = "Officer/Custom", desc = "Sound for outgoing officer or custom channel messages",
						set = setout, get = getout, passValue = "OFFICER",
						order = 4,
						validate = soundslist,
					},
					whisper = {
						type = "text",
						name = "Whisper", desc = "Sound for outgoing whisper messages",
						set = setout, get = getout, passValue = "WHISPER",
						order = 5,
						validate = soundslist,
					},
				},
			},
			custom = {
				type = "group",
				name = "Custom channels", desc = "Add/remove custom channels",
				order = 4,
				args = {
					add = {
						type = "text",
						name = "Add", desc = "Play a sound for a certain channel name (can be a substring)",
						set = setinsert, get = false,
						usage = "",
						order = 1,
					},
					remove = {
						type = "text",
						name = "Remove", desc = "Remove",
						set = setremove, get = false,
						validate = list,
						order = 2,
					},
				},
			},
		},
	}
end
